# Default and sample prompt for the tool agent
prompt = """
# Tool Specification Generator Prompt

You are a tool specification generator. You are given a table schema and sample data.
You need to generate a tool specification for the table.
The tool specification should be in the format of a JSON object.
While generate descriptions, define it in a way to make it easier for LLM to understand it.
Use the sample data to identify data types, patterns and enums.

IMPORTANT: Consider ONLY the partition keys, sorting keys and indexed columns as parameteres.
IMPORTANT: Partition keys are mandatory parameters.
IMPORTANT: For indexed date time or timestamps parameters, generate start_<column_name> and end_<column_name> parameters. use $gt and $lte operators.
IMPORTANT: For indexed numeric parameters, generate min_<column_name> and max_<column_name> parameters. use $gte and $lte operators.
IMPORTANT: Return ONLY valid JSON without any markdown formatting, code blocks, or additional text.
IMPORTANT: If the column is a vector column, generate the embedding_model as text-embedding-3-small.

The table schema is: {metadata}

The sample data is: {sample_records}

Additional instructions: {additional_instructions}

Generate the tool specification in the following format:
{{
    "tags": [<main topics of the data>],
    "type": "tool",
    "name": <name of the tool - String>,
    "description": <description of the tool - String>,
    "projection": < relevant fields to return. avoid codes, and technical fields like _id, _createdAt, _updatedAt, _deletedAt, etc. E.g: {{"column_name": 1, "column_name2": 1, "column_name3": 1}} | type: Object | default: {{}}>,
    "parameters": [
        {{
            "param": <name of the parameter - String | type: String >
            "description": <instruction to the LLM about how to use the parameter - String>,
            "attribute": <name of the column in the table | type: String | default: $vectorize | If it is equal to the param, do not fill this field>
            "type": <type of the parameter according to the attribute>,
            "required": <required parameter. If the attribute is partition key, it is mandatory | type: Boolean | default: False>,
            "operator": < The operator to use to filter the parameter - if not filled, the operator is $eq | type: String | default: $eq | If the attribute is not a vector column, do not fill this field>,
            "enum": <enum of the parameter - Array of Strings | If no enum detected, do not fill this field>,
            "embedding_model": <embedding model of the parameter - String | If no embedding model detected, do not fill this field>,
            "expr": <if theres a expression for the parameters, like filter conditions, add it here. Use python basic operations or datetime operations | type: String | default: None | If unknown, do not fill this field>,
            "value": <if theres a static value for the parameters, like filter conditions, add it here | type: Any | default: None | If unknown, do not fill this field>,
            "info": <inform if the attribute is part of partitionk key, sorting key, indexed column or vector column | type: String | default: "">
        }},
    ],
    "method": "find",
    "table_name": <table name of the tool>,
    "db_name": <db name of the tool>,
    "limit": <limit of the tool | Default: 10>
}}"""



from fastmcp.server.middleware import Middleware, MiddlewareContext
from fastmcp.exceptions import ToolError
from fastmcp.tools.tool import ToolResult
from .logger import get_logger
import mcp.types as types
import json
from .database import AstraDBManager
import os
from datetime import datetime # for datetime eval expressions that can be used in the tool config
import uuid

class AuditStatus:
    STARTED = "started"
    COMPLETED = "completed"
    FAILED = "failed"

class RunToolMiddleware(Middleware):
    
    logger = get_logger("RunToolMiddleware")

    def __init__(self, astra_db_manager: AstraDBManager, tools_config: dict):
        self.astra_db_manager = astra_db_manager
        self.tools_config = tools_config

    async def on_call_tool(self, context: MiddlewareContext, call_next):
        # Access the tool object to check its metadata
        if not context.fastmcp_context:
            ToolError("No context found")

        tool_name = context.message.name
        run_id = uuid.uuid1() # Use UUID1 for timestamp based UUID
        start_timestamp = datetime.now().isoformat()
        
        self.logger.info(f"Run ID: {run_id}")
        self.logger.info(f"Start timestamp: {start_timestamp}")
        self.logger.info(f"Context: {context}")
        self.astra_db_manager.log_audit(tool_id=tool_name, 
                                       client_id=context.fastmcp_context.client_id, 
                                       run_id=run_id, 
                                       start_timestamp=start_timestamp,
                                       status=AuditStatus.STARTED)
        
        try:
            arguments = context.message.arguments
            self.logger.debug(f"Arguments: {arguments}")
            self.astra_db_manager.log_audit(tool_id=tool_name, 
                                       run_id=run_id, 
                                       parameters= json.dumps(arguments),
                                       status=AuditStatus.STARTED)
        except Exception as e:
            self.logger.error(f"Error getting arguments: {e}")
            self.astra_db_manager.log_audit(tool_id=tool_name, 
                                       run_id=run_id, 
                                       end_timestamp=datetime.now().isoformat(),
                                       status=AuditStatus.FAILED,
                                       status_code=500,
                                       status_message=f"Error getting arguments: {e}",
                                       status_details=str(e),
                                       error=str(e))
            return ToolResult({"error": f"Error getting arguments: {e}"})

        tool_config = next(
            (t for t in self.tools_config if t['name'] == tool_name), None)

        if not tool_config:
            ToolError(f"Tool {tool_name} not found")
        else:
            self.logger.debug(f"Tool config: {tool_config}")

        # Check arguments            
        for param in tool_config["parameters"]:
            if param["param"] not in arguments and param.get("required", False) == True:
                self.logger.error(f"Parameter {param['param']} is required")
                return ToolResult({"error": f"Parameter {param['param']} is required"})

        # Run methods
        if tool_config["method"] == "find" or tool_config["method"] == "find_documents":
            result = self.astra_db_manager.find(
                arguments=arguments,
                tool_config=tool_config)

            self.logger.debug(f"Result: {result}")
            self.astra_db_manager.log_audit(tool_id=tool_name, 
                                       run_id=run_id, 
                                       end_timestamp=datetime.now().isoformat(),
                                       status=AuditStatus.COMPLETED,
                                       status_code=200)
            return ToolResult(structured_content=result)

        if tool_config["method"] == "list_collections":
            result = self.astra_db_manager.list_collections()
            self.logger.debug(f"Result: {result}")
            return ToolResult(structured_content=result)
        
        # Method not implemented
        ToolError(f"Method {tool_config['method']} not allowed")



import time
from typing import Any
from fastmcp.server.auth.providers.jwt import AccessToken, TokenVerifier
from .logger import get_logger
import os
logger = get_logger("auth")

class AstraAuth(TokenVerifier):
    """
    Astra auth for testing and development.
    """

    def __init__(
        self,
        token: str
    ):
        """
        Initialize the Astra auth.

        Args:
            token: Dict mapping token strings to token metadata
                   Each token should have: client_id, scopes, expires_at (optional)
        """
        super().__init__()
        self.token = token

    async def verify_token(self, token: str) -> AccessToken | None:
        """Verify token against Astra token dictionary."""
        logger.info(f"Token: {token}")
        
        token_env = os.getenv("AGENTIC_ASTRA_TOKEN") or os.getenv("ASTRA_MCP_SERVER_TOKEN")
        if token != token_env:
            return None
        logger.info(f"Token successfully verified: {token}")
        
        return AccessToken(
            token=token,
            client_id="agentic-astra",
            scopes=["read:data"],
            expires_at=time.time() + 3600,
            claims={},
        )


# test_mcp_connection.py
import pytest
from fastmcp import Client
import pytest_asyncio
from fastmcp.client.transports import StreamableHttpTransport
from fastmcp.client.auth import BearerAuth
import os
from dotenv import load_dotenv
load_dotenv(override=True)

@pytest_asyncio.fixture
async def mcp_client():
    """Fixture to create an MCP client connected to the server."""
    transport = StreamableHttpTransport(
    url="http://localhost:8000/mcp",
    headers={
        "Authorization": "Bearer " + (os.getenv("AGENTIC_ASTRA_TOKEN") or os.getenv("ASTRA_MCP_SERVER_TOKEN"))
        }
    )
    
    client = Client(transport)
    async with client:
        yield client

@pytest.mark.asyncio
async def test_server_connection(mcp_client):
    """Test that the MCP client can successfully ping the server."""
    response = await mcp_client.ping()  # assuming the client has a ping method
    assert response == True, "Server did not respond with 'ok'"

@pytest.mark.asyncio
async def test_list_tools(mcp_client):
    """Test listing tools through the MCP client."""
    result = await mcp_client.list_tools()
    print(result)
    assert result is not None, "Server did not respond with tools"
    assert len(result) > 0, "Server did not respond with tools"

@pytest.mark.asyncio
async def test_run_search_airline_tickets(mcp_client):
    """Test running the search_products tool through the MCP client."""
    result = await mcp_client.call_tool("search_airline_tickets", {"customer_id": "dc3a050c-0eb5-42dd-ab4f-450f0819c8e4"})
    print(result)
    assert result is not None, "Server did not respond with search_airline_tickets tool"
    assert result.documents is not None, "Server did not respond with search_products tool"
    


from math import e
from .database import AstraDBManager
from .logger import get_logger
import os
import json
import argparse
from dotenv import load_dotenv
from .utils import add_underscore_to_dict_keys
load_dotenv(override=True)

class AstraCatalog:
    logger = get_logger("catalog")
    
    def __init__(self):
        self.astra_db_name = os.getenv("ASTRA_DB_DB_NAME")
        self.astra_db_manager = AstraDBManager(os.getenv("ASTRA_DB_APPLICATION_TOKEN"), os.getenv("ASTRA_DB_API_ENDPOINT"), self.astra_db_name)
        self.db = self.astra_db_manager.get_db_by_name(self.astra_db_name)
    
    def upload_catalog(self, catalog: dict, table_name: str):
        collection_name = table_name
        
        list_collections = self.db.list_collection_names()
        self.logger.info(f"List of collections: {list_collections}")
        
        if collection_name not in list_collections:
            self.logger.info(f"Collection {collection_name} not found, creating it")
            self.db.create_collection(collection_name)
            self.logger.info(f"Collection {collection_name} created")
        
        collection = self.db.get_collection(collection_name)
            
        self.logger.info(f"Deleting all catalog items from {collection_name}")
        
        if not isinstance(catalog, list):
            catalog = [catalog]
            
        tools_to_update = []
        for tool in catalog:
            if tool["type"] == "tool":
                tools_to_update.append(tool["name"])
        self.logger.info(f"Tools to update: {tools_to_update}")        
        collection.delete_many({"name": {"$in": tools_to_update}})
        self.logger.info(f"Uploading {len(catalog)} catalog items to {collection_name}")
        result = collection.insert_many(catalog)
        self.logger.info(f"Uploaded {len(result.inserted_ids)} catalog items")
        return result.inserted_ids    

def upload_catalog(file_path: str, table_name: str):
    catalog = AstraCatalog()
    catalog_content = json.load(open(file_path))
    catalog_content = add_underscore_to_dict_keys(catalog_content)
    catalog.upload_catalog(catalog_content, table_name)

def main():
    parser = argparse.ArgumentParser(description="Astra MCP Server Catalog Uploader")
    parser.add_argument("-f", "--file_path", help="Path to the catalog file")
    parser.add_argument("-t", "--table_name", default=os.getenv("ASTRA_DB_CATALOG_COLLECTION") or "tool_catalog", help="Table name")

    args = parser.parse_args()
    upload_catalog(args.file_path, args.table_name)

if __name__ == "__main__":
    main()



from astrapy.constants import SortMode
from astrapy.info import (
    CreateTableDefinition,
    ColumnType,
    TableScalarColumnTypeDescriptor,
    TableValuedColumnTypeDescriptor,
    TableValuedColumnType,
    TablePrimaryKeyDescriptor,
)


audit_table_definition = CreateTableDefinition(
    columns={
        "tool_id": TableScalarColumnTypeDescriptor(column_type=ColumnType.TEXT),
        "date": TableScalarColumnTypeDescriptor(column_type=ColumnType.TEXT),
        "run_id": TableScalarColumnTypeDescriptor(column_type=ColumnType.UUID),
        "client_id": TableScalarColumnTypeDescriptor(column_type=ColumnType.TEXT),
        "start_timestamp": TableScalarColumnTypeDescriptor(column_type=ColumnType.TIMESTAMP),
        "end_timestamp": TableScalarColumnTypeDescriptor(column_type=ColumnType.TIMESTAMP),
        "keys": TableValuedColumnTypeDescriptor(
            column_type=TableValuedColumnType.SET,
            value_type=ColumnType.TEXT,
        ),
        "parameters": TableScalarColumnTypeDescriptor(column_type=ColumnType.TEXT),
        "result": TableScalarColumnTypeDescriptor(column_type=ColumnType.TEXT),
        "error": TableScalarColumnTypeDescriptor(column_type=ColumnType.TEXT),
        "status": TableScalarColumnTypeDescriptor(column_type=ColumnType.TEXT),
        "status_code": TableScalarColumnTypeDescriptor(column_type=ColumnType.INT),
        "status_message": TableScalarColumnTypeDescriptor(column_type=ColumnType.TEXT),
        "status_details": TableScalarColumnTypeDescriptor(column_type=ColumnType.TEXT),
    },
    primary_key=TablePrimaryKeyDescriptor(partition_by=["tool_id", "date"], 
                                          partition_sort={"run_id": SortMode.DESCENDING}),
)


import re
from typing import Any
import os
# Define development tokens and their associated claims

ALLOWED_ENV_VARIABLES = ["ASTRA_DB_APPLICATION_TOKEN", "ASTRA_DB_API_ENDPOINT", "ASTRA_DB_DB_NAME", "OPENAI_API_KEY", "LOG_LEVEL", "LOG_FILE", "ASTRA_DB_CATALOG_COLLECTION"]
def load_env_variables(env_args, logger):
    """
    Load environment variables from command line arguments.
    
    Args:
        env_args: List of strings in KEY=VALUE format
    """
    if not env_args:
        return
    
    for env_var in env_args:
        if "=" not in env_var:
            logger.warning(f"Invalid environment variable format: {env_var}. Expected KEY=VALUE")
            continue
        
        key, value = env_var.split("=", 1)  # Split only on first '=' to handle values with '='
        if key not in ALLOWED_ENV_VARIABLES:
            logger.warning(f"Invalid environment variable: {key}. Allowed variables are: {ALLOWED_ENV_VARIABLES}")
            continue
        os.environ[key] = value
        logger.info(f"Loaded environment variable: {key}")

def extract_db_id_from_astra_url(url: str) -> str:
    """
    Extract UUID from Astra URL.
    
    Args:
        url: Astra URL in format https://uuid-region.apps.astra.datastax.com
    
    Returns:
        The UUID string (e.g., "4dcd68d4-978d-45c6-a6b5-505416f98a61")
    """
    # Pattern to match UUID followed by region in Astra URL
    pattern = r'https://([a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12})-'
    match = re.search(pattern, url)
    
    if match:
        return match.group(1)
    else:
        raise ValueError(f"Could not extract UUID from URL: {url}")


def remove_underscore_from_dict_keys(data: Any) -> Any:
    """
    Recursively rename dictionary keys by replacing '_$' with '$' only when it starts the key.
    
    Args:
        data: The data structure to process (dict, list, or other)
    
    Returns:
        The data structure with renamed keys
    """
    if isinstance(data, dict):
        # Create a new dictionary with renamed keys
        new_dict = {}
        for key, value in data.items():
            # Replace '_$' with '$' only if the key starts with '_$'
            if isinstance(key, str) and key.startswith("_$"):
                new_key = key[1:]  # Remove the first character '_'
            else:
                new_key = key
            # Recursively process the value
            new_dict[new_key] = remove_underscore_from_dict_keys(value)
        return new_dict
    elif isinstance(data, list):
        # Process each item in the list
        return [remove_underscore_from_dict_keys(item) for item in data]
    else:
        # Return primitive values as-is
        return data


def add_underscore_to_dict_keys(data: Any) -> Any:
    """
    Recursively rename dictionary keys by adding '_' before '$' only when the key starts with '$'.
    
    Args:
        data: The data structure to process (dict, list, or other)
    
    Returns:
        The data structure with renamed keys
    """
    if isinstance(data, dict):
        # Create a new dictionary with renamed keys
        new_dict = {}
        for key, value in data.items():
            # Add '_' before '$' only if the key starts with '$'
            if isinstance(key, str) and key.startswith("$"):
                new_key = "_" + key  # Add underscore before the dollar sign
            else:
                new_key = key
            # Recursively process the value
            new_dict[new_key] = add_underscore_to_dict_keys(value)
        return new_dict
    elif isinstance(data, list):
        # Process each item in the list
        return [add_underscore_to_dict_keys(item) for item in data]
    else:
        # Return primitive values as-is
        return data


import os
import requests

EMBEDDING_PROVIDER = {
    "text-embedding-3-small": "openai",
    "text-embedding-3-large": "openai",
    "text-embedding-ada-002": "openai",
    "all-minilm-l6-v2": "ibm-watsonx",
    "granite-embedding-107m-multilingual": "ibm-watsonx",
    "granite-embedding-278m-multilingual": "ibm-watsonx",
    "ms-marco-minilm-l-12-v2": "ibm-watsonx",
    "multilingual-e5-large": "ibm-watsonx",
    "slate-30m-english-rtrvr-v2": "ibm-watsonx",
    "slate-30m-english-rtrvr": "ibm-watsonx",
    "slate-125m-english-rtrvr-v2": "ibm-watsonx",
    "slate-125m-english-rtrvr": "ibm-watsonx"
}


def run_prompt(prompt: str) -> str:
    base_url = os.getenv("OPENAI_BASE_URL") or "https://api.openai.com/v1"
    api_key = os.getenv("OPENAI_API_KEY")

    if not base_url or not api_key:
        raise EnvironmentError("Missing OPENAI_BASE_URL or OPENAI_API_KEY environment variables.")

    url = f"{base_url.rstrip('/')}/chat/completions"
    headers = {
        "Content-Type": "application/json",
        "Authorization": f"Bearer {api_key}",
    }
    payload = {
        "model": "gpt-4o-mini",
        "messages": [{"role": "user", "content": prompt}],
    }
    response = requests.post(url, headers=headers, json=payload)
    response.raise_for_status()
    return response.json()
    
def generate_embedding(text: str, model: str = "text-embedding-3-small") -> list[float]:
    if EMBEDDING_PROVIDER[model] == "openai":
        return generate_embedding_openai(text, model)
    elif EMBEDDING_PROVIDER[model] == "ibm-watsonx":
        return generate_embedding_ibm_watsonx(text, model)
    else:
        raise ValueError(f"Unsupported embedding model: {model}")

def generate_embedding_ibm_watsonx(text: str, model: str = "granite-embedding-278m-multilingual") -> list[float]:
    """ 
    Generate an embedding using the IBM Watsonx REST API without using the SDK.
    Environment variables required:
      - IBM_WATSONX_BASE_URL: the base URL of the IBM Watsonx endpoint (e.g., https://{cluster_url}/ml/v1/text/embeddings?version=2023-10-25)
      - IBM_WATSONX_API_KEY: your API key
      - IBM_WATSONX_PROJECT_ID: your project ID
    """
    base_url = os.getenv("IBM_WATSONX_BASE_URL")
    api_key = os.getenv("IBM_WATSONX_API_KEY")
    project_id = os.getenv("IBM_WATSONX_PROJECT_ID")

    if not base_url or not api_key or not project_id:
        raise EnvironmentError("Missing IBM_WATSONX_BASE_URL or IBM_WATSONX_API_KEY or IBM_WATSONX_PROJECT_ID environment variables.")

    url = f"{base_url.rstrip('/')}"

    headers = {
        "Content-Type": "application/json",
        "Authorization": f"Bearer {api_key}",
    }

    payload = {
        "model_id": model,
        "inputs": [text],
        "project_id": os.getenv("IBM_WATSONX_PROJECT_ID")
    }

    response = requests.post(url, headers=headers, json=payload)
    response.raise_for_status()

    data = response.json()
    return data["results"][0]["embedding"]

def generate_embedding_openai(text: str, model: str = "text-embedding-3-small") -> list[float]:
    """
    Generate an embedding using the OpenAI REST API without using the SDK.

    Environment variables required:
      - OPENAI_API_KEY: your API key
      - OPENAI_BASE_URL: the base URL of the OpenAI endpoint (e.g., https://api.openai.com/v1)
    """
    base_url = os.getenv("OPENAI_BASE_URL") or "https://api.openai.com/v1"
    api_key = os.getenv("OPENAI_API_KEY")

    if not base_url or not api_key:
        raise EnvironmentError("Missing OPENAI_BASE_URL or OPENAI_API_KEY environment variables.")

    url = f"{base_url.rstrip('/')}/embeddings"

    headers = {
        "Content-Type": "application/json",
        "Authorization": f"Bearer {api_key}",
    }

    payload = {
        "model": model,
        "input": text,
    }

    response = requests.post(url, headers=headers, json=payload)
    response.raise_for_status()

    data = response.json()
    return data["data"][0]["embedding"]





#!/usr/bin/env python3
"""
Test script to verify logging configuration.
This script tests that logs go only to the file and stdout remains available.
"""

import sys
from logger import get_logger

def test_logging():
    """Test the logging configuration."""
    print("=== Testing Logging Configuration ===")
    print("This message should appear on stdout (not in log file)")
    
    # Get logger instance
    logger = get_logger("test_logger")
    
    # Test different log levels
    logger.debug("This is a DEBUG message - should go to logs/logs.log only")
    logger.info("This is an INFO message - should go to logs/logs.log only")
    logger.warning("This is a WARNING message - should go to logs/logs.log only")
    logger.error("This is an ERROR message - should go to logs/logs.log only")
    
    print("=== Test completed ===")
    print("Check logs/logs.log file to verify the log messages are there")
    print("This stdout message should NOT be in the log file")

if __name__ == "__main__":
    test_logging()



"""
Logging configuration for Astra MCP Server

Provides centralized logging setup with different log levels and formats.
"""

import logging
import sys
import os
from datetime import datetime
from typing import Optional

class LoggerConfig:
    """Configuration class for logging setup."""
    
    def __init__(
        self,
        name: str = "agentic_astra",
        level: str = "INFO",
        log_file: Optional[str] = "logs/logs.log",
        format_string: Optional[str] = None,
        stdout: bool = False
    ):
        self.name = name
        self.level = getattr(logging, level.upper(), logging.INFO)
        self.log_file = log_file
        self.format_string = format_string or "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
        self.stdout = stdout
    def setup_logger(self) -> logging.Logger:
        """Set up and configure the logger."""
        logger = logging.getLogger(self.name)
        logger.setLevel(self.level)
        
        # Clear existing handlers to avoid duplicates
        logger.handlers.clear()
        
        # Create formatter
        formatter = logging.Formatter(self.format_string)
        
        # Console handler removed - all logs go to file only
        # console_handler = logging.StreamHandler(sys.stdout)
        # console_handler.setLevel(self.level)
        # console_handler.setFormatter(formatter)
        # logger.addHandler(console_handler)
        
        # File handler (if log_file is specified)
        if self.log_file:
            # Create logs directory if it doesn't exist
            log_dir = os.path.dirname(self.log_file)
            if log_dir and not os.path.exists(log_dir):
                os.makedirs(log_dir)
                
            file_handler = logging.FileHandler(self.log_file)
            file_handler.setLevel(self.level)
            file_handler.setFormatter(formatter)
            logger.addHandler(file_handler)

        if self.stdout:
            console_handler = logging.StreamHandler(sys.stdout)
            console_handler.setLevel(self.level)
            console_handler.setFormatter(formatter)
            logger.addHandler(console_handler)
        
        return logger

def get_logger(
    name: str = "agentic_astra",
    level: str = None,
    log_file: Optional[str] = None,
    stdout: bool = False
) -> logging.Logger:
    """
    Get a configured logger instance.
    
    Args:
        name: Logger name
        level: Log level (DEBUG, INFO, WARNING, ERROR, CRITICAL)
        log_file: Optional log file path
    
    Returns:
        Configured logger instance
    """
    # Use environment variable for log level if not specified
    if level is None:
        level = os.getenv("LOG_LEVEL", "INFO")
    
    # Use environment variable for log file if not specified
    if log_file is None:
        log_file = os.getenv("LOG_FILE")
    
    config = LoggerConfig(name=name, level=level, log_file=log_file, stdout=stdout)
    return config.setup_logger()

# Create default logger instance
logger = get_logger()

def log_function_call(func_name: str, args: dict = None, kwargs: dict = None):
    """Decorator helper to log function calls."""
    def decorator(func):
        def wrapper(*args, **kwargs):
            logger.debug(f"Calling {func_name} with args: {args}, kwargs: {kwargs}")
            try:
                result = func(*args, **kwargs)
                logger.debug(f"{func_name} completed successfully")
                return result
            except Exception as e:
                logger.error(f"{func_name} failed with error: {str(e)}")
                raise
        return wrapper
    return decorator



"""
Astra DB Operations

Database operations and utilities for interacting with Astra DB.
"""

import asyncio
import json
from typing import Any, Dict, List, Optional
from dataclasses import dataclass

from dotenv import load_dotenv
from astrapy import DataAPIClient
from astrapy.data_types import DataAPIVector, DataAPITimestamp
from .logger import get_logger
from .utils import remove_underscore_from_dict_keys, extract_db_id_from_astra_url
from .llm import generate_embedding
from .audit import audit_table_definition
from datetime import datetime

# Load environment variables
load_dotenv()

# Initialize logger

@dataclass
class AppContext:
    """Application context containing Astra DB connection."""
    db: Optional[Any] = None  # Database

class AstraDBManager:
    """Manager class for Astra DB operations."""
    logger = get_logger("Astra DB Manager")
    
    def __init__(self, token: str, endpoint: str = None, db_name: str = None):
        self.astra_db_token = token
        self.astra_db_api_endpoint = endpoint
        self.astra_db_db_name = db_name
        self.client = None
        self.db = {}
        self._initialize_database()
    
    def _initialize_database(self):
        """Initialize Astra DB connection."""
        
        if not self.astra_db_token or (not self.astra_db_db_name and not self.astra_db_api_endpoint):
            self.logger.error("Astra DB credentials not found. Set ASTRA_DB_APPLICATION_TOKEN and ASTRA_DB_API_ENDPOINT or ASTRA_DB_DB_NAME environment variables.")
            return
        
        try:
            self.client = DataAPIClient()
            # to keep compatibility with the old version
            if not self.astra_db_db_name:
                db_list = self.get_dbs()
                if self.astra_db_api_endpoint:
                    catalog_db_id = extract_db_id_from_astra_url(self.astra_db_api_endpoint)
                    catalog_db = next((db for db in db_list if db.id == catalog_db_id), None)    
                    self.astra_db_db_name = catalog_db.name
                else:
                    # If no db name or api endpoint, use the first db
                    self.astra_db_db_name = db_list[0].name
                
            self.logger.info("Connected to Astra DB successfully")
        except Exception as e:
            self.logger.error(f"Could not connect to Astra DB: {e}")
    
    def get_db_by_name(self, db_name: str):
        if db_name not in self.db:
            db_list = self.get_dbs()
            self.logger.debug("db_list: %s", db_list)
            new_db = next((db for db in db_list if db.name == db_name), None)
            if not new_db:
                self.logger.error(f"Database {db_name} not found.")
                return
            
            self.logger.debug("new_db: %s", new_db)
            
            self.db[db_name] = self.client.get_database(
                new_db.regions[0].api_endpoint,
                token=self.astra_db_token
            )
            
        return self.db[db_name]
    
    def get_dbs(self) -> [Any]:
        admin_client = self.client.get_admin(token=self.astra_db_token)
        return admin_client.list_databases()

    def get_catalog_content(self, collection_name: str, tags: Optional[str] = None) -> str:
        """Get catalog content from Astra DB collection."""
        db = self.get_db_by_name(self.astra_db_db_name)
        self.logger.info(f"Database: {db}")
        collection =  db.get_collection(collection_name)
        self.logger.info(f"Getting catalog content from {collection_name} with tags: {tags}")
        result = None
        if tags:
            result = collection.find({"type": "tool", "tags": {"$in": tags.split(",")}})
        else:
            result = collection.find({"type": "tool",})
        result = remove_underscore_from_dict_keys(list(result))
        return result
    
    def setup_audit_trail(self, table_name: str):
        """Setup audit trail for the database."""
        db = self.get_db_by_name(self.astra_db_db_name)
        
        tables = db.list_table_names()
        
        if table_name not in tables:
            self.logger.info(f"Creating audit table {table_name}")  
            self.audit_table  = db.create_table(table_name, definition = audit_table_definition)
            self.logger.info(f"Audit table {table_name} created")
        else:
            self.logger.info(f"Audit table {table_name} already exists")
            self.audit_table = db.get_table(table_name)
    
    def log_audit(self, 
                  tool_id: str, 
                  run_id: str, 
                  client_id: Optional[str] = None, 
                  start_timestamp: str = None, 
                  end_timestamp: str = None,
                  keys: List[str] = None, 
                  parameters: Dict[str, Any] = None, 
                  result: Dict[str, Any] = None, 
                  error: Optional[str] = None, 
                  status: Optional[str] = None, 
                  status_code: Optional[int] = None, 
                  status_message: Optional[str] = None, 
                  status_details: Optional[str] = None):
        """Log audit trail for the database."""
        if not self.audit_table:
            return
        
        if start_timestamp:
            start_timestamp = datetime.strptime(start_timestamp, "%Y-%m-%dT%H:%M:%S.%f")
        if end_timestamp:
            end_timestamp = datetime.fromisoformat(end_timestamp)
            
        self.logger.info(f"Start timestamp: {start_timestamp}")
        self.logger.info(f"End timestamp: {end_timestamp}")
            
        payload = {
            "tool_id": tool_id,
            "client_id": client_id,
            "run_id": run_id,
            "date": datetime.now().strftime("%Y-%m-%d"),
            "start_timestamp":  DataAPITimestamp.from_datetime(start_timestamp) if start_timestamp else None,
            "end_timestamp": DataAPITimestamp.from_datetime(end_timestamp) if end_timestamp else None,
            "keys": keys,
            "parameters": parameters,
            "result": result,
            "error": error,
            "status": status,
            "status_code": status_code,
            "status_message": status_message,
            "status_details": status_details,
        }
        
        # Remove None values from payload
        payload = {k: v for k, v in payload.items() if v is not None}
        self.logger.debug(f"Payload: {payload}")
        self.logger.debug(f"Inserting audit trail for {tool_id} with payload: {payload}")
        self.audit_table.insert_one(payload)
        self.logger.debug(f"Audit trail for {tool_id} inserted successfully")

    def find(
        self,
        arguments: Optional[Dict[str, Any]] = None,
        tool_config: Optional[Dict[str, Any]] = None,
    ) -> Dict[str, Any]:
        """
        Find documents in Astra DB collection.
        """
        try:
            if not tool_config:
                self.logger.error("Tool config not found")
                return json.dumps({"error": "Tool config not found"})
            
            # Where to run the query
            object_type = "collection" if "collection_name" in tool_config else "table"
            object_name = tool_config["collection_name"] if "collection_name" in tool_config else tool_config["table_name"]
            db_name = tool_config["db_name"] if "db_name" in tool_config else self.astra_db_db_name
            
            self.logger.debug(f"Finding documents in '{object_type}' '{object_name}' in database '{db_name}'")
            
            db = self.get_db_by_name(db_name)
            target_object = None

            if object_type == "collection":
                target_object = db.get_collection(object_name)
            else:
                target_object = db.get_table(object_name)
                
            if not target_object:
                self.logger.error(f"{object_type} '{object_name}' not available.")
                return json.dumps({"error": f"{object_type} '{object_name}' not available."})
            
            filter_dict = {}
            search_query = None
            for param in tool_config["parameters"]:
                
                attribute = param["attribute"] if "attribute" in param else param["param"]

                if attribute == "$vector" or attribute == "$vectorize":
                    search_query = arguments[param["param"]]
                    continue

                operator = "$eq"
                if "operator" in param:
                    operator = param["operator"]

                if "value" in param:
                    filter_dict[attribute] = {operator: param["value"]}
                elif "expr" in param:
                    filter_dict[attribute] = eval(param["expr"])
                elif param["param"] in arguments:
                    filter_dict[attribute] = {
                        operator: arguments[param["param"]]}
                    
            # Find parameters
            find_params = {}
            
            if filter_dict:
                find_params["filter"] = filter_dict
                
            if tool_config["limit"]:
                find_params["limit"] = tool_config["limit"]
            
            if search_query:
                search_query_config = next((p for p in tool_config["parameters"] if p["param"] == "search_query"), None)
                if "embedding_model" in search_query_config:
                    try:    
                        embedding = generate_embedding(search_query, search_query_config["embedding_model"])
                        find_params["sort"] = {"$vector": DataAPIVector(embedding)}
                    except Exception as e:
                        self.logger.error(f"Failed to generate embedding: {str(e)}")
                        return json.dumps({"error": f"Failed to generate embedding: {str(e)}"})
                elif search_query_config["attribute"] == "$vectorize":
                    find_params["sort"] = {"$vectorize": search_query}
                else:
                    self.logger.error("Search query attribute must be $vectorize or $vector")
                    return json.dumps({"error": "Search query attribute must be $vectorize or $vector"})
            
            elif "sort" in tool_config:
                find_params["sort"] = tool_config["sort"]
            
            if "projection" in tool_config:
                find_params["projection"] = tool_config["projection"]
            
            self.logger.debug("find_params %s", find_params)

            result = target_object.find(**find_params)
            documents = list(result)
            self.logger.info(f"Found {len(documents)} documents in {object_type} '{object_name}'")
            return {
                "success": True,
                "count": len(documents),
                "documents": documents
            }
        except Exception as e:
            self.logger.error(f"Failed to find documents in {object_type} '{object_name}': {str(e)}")
            return json.dumps({"error": f"Failed to find documents: {str(e)}"})

    def list_collections(self = None) -> str:
        """
        List all collections in the Astra DB database.
        """
        self.logger.debug("Listing all collections in Astra DB")
        
        try:
            db = self.get_db_by_name(self.astra_db_db_name)
            collections = db.list_collection_names()
            self.logger.info(f"Found {len(collections)} collections: {collections}")
            return json.dumps({
                "success": True,
                "collections": collections
            })
        except Exception as e:
            self.logger.error(f"Failed to list collections: {str(e)}")
            return json.dumps({"error": f"Failed to list collections: {str(e)}"})



#!/usr/bin/env python3
"""
Tool Specification Generator for Astra DB Tables

This program connects to Astra DB, retrieves table schema and sample data,
and generates a tool specification in the format compatible with tool_config.json.
"""

import argparse
import json
import os
import sys
from typing import Any, Dict, List, Optional
from dotenv import load_dotenv
from pydantic_core.core_schema import none_schema
from astrapy.info import TableIndexType
# Add the parent directory to the path to import agentic_astra modules
sys.path.insert(0, os.path.dirname(os.path.dirname(__file__)))

from agentic_astra.database import AstraDBManager
from agentic_astra.logger import get_logger
from agentic_astra.llm import run_prompt
from agentic_astra.tool_agent_prompt import prompt as tool_agent_prompt

# Load environment variables
load_dotenv(override=True)

logger = get_logger("Agent Tool Generator", stdout=True)

def export_prompt(file_path: str) -> str:
    prompt = tool_agent_prompt
    with open(file_path, "w") as f:
        f.write(prompt)
    logger.info(f"Prompt exported to {file_path}")
class AstraToolAgent:
    
    db_manager = None
    db = None
    table = None
    collection = None
    table_name = None
    collection_name = None
    keyspace_name = None
    metadata = {}
    def __init__(self, astra_db_token: str, astra_db_name: str, table_name: str = None, collection_name: str = None, keyspace_name: str = "default_keyspace") -> Optional[Dict[str, Any]]:
        """Initialize the tool agent."""
        self.db_manager =  AstraDBManager(token=astra_db_token, db_name=astra_db_name)
        self.db = self.db_manager.get_db_by_name(astra_db_name)
        self.keyspace_name = keyspace_name
        if table_name:
            self.table_name = table_name
            self.table = self.db.get_table(self.table_name)
            

            
    def get_indexed_columns(self, table_indexes: List[Dict[str, Any]], index_type: TableIndexType = TableIndexType.REGULAR) -> List[str]:
        indexed_columns = []
        for index in table_indexes:
            if index.index_type == index_type:
                indexed_columns.append(index.definition.column)
        return indexed_columns

    def get_table_schema(self) -> Optional[Dict[str, Any]]:
        """Get table schema information from Astra DB."""
        tables = self.db.list_tables()        
        table_metadata = next((table for table in tables if table.name == self.table_name), None)
        table_indexes = self.table.list_indexes()
        
        
        metadata = {
            "table_name": self.table_name,
            "keyspace_name": self.keyspace_name,
            "columns": table_metadata.raw_descriptor["definition"]["columns"],
            "primary_key": table_metadata.raw_descriptor["definition"]["primaryKey"],
            "partition_key": table_metadata.raw_descriptor["definition"]["primaryKey"]["partitionBy"],
            "partition_sort": table_metadata.raw_descriptor["definition"]["primaryKey"]["partitionSort"],
            "indexed_columns": self.get_indexed_columns(table_indexes, TableIndexType.REGULAR),
            "vector_columns": self.get_indexed_columns(table_indexes, TableIndexType.VECTOR),
            "text_columns": self.get_indexed_columns(table_indexes, TableIndexType.TEXT)
        }
        logger.info(f"Table metadata: {metadata}")
        
        logger.info(f"Retrieved schema for table '{self.table_name}': {metadata}")
        return metadata
        
    def get_sample_records(self, limit: int = 5) -> Optional[List[Dict[str, Any]]]:
        """Get sample records from the table."""
        try:
            # Get first few records
            result = self.table.find(limit=limit)
            records = list(result)
            logger.info(f"Retrieved {len(records)} sample records from table '{self.table.name}'")
            return records
            
        except Exception as e:
            logger.error(f"Failed to get sample records from table '{self.table.name}': {str(e)}")
            return None

    def load_prompt_template(self, prompt_file: str = None) -> str:
        """Load the prompt template from the markdown file."""
        if prompt_file:
            prompt_file_path = prompt_file
        else:
            prompt_file_path = os.path.join(os.path.dirname(__file__), "tool_specification_prompt.md")
        try:
            with open(prompt_file_path, 'r', encoding='utf-8') as f:
                return f.read()
        except FileNotFoundError:
            logger.error(f"Prompt template file not found: {prompt_file_path}")
            raise
        except Exception as e:
            logger.error(f"Error reading prompt template file: {e}")
            raise

    def generate_tool_specification(self, sample_size: int = 5, additional_instructions: str = "", prompt_file: str = None) -> Dict[str, Any]:
        """Generate tool specification based on table schema and sample data."""
        self.metadata = self.get_table_schema()
        sample_records = self.get_sample_records(limit=sample_size)
        
        # Load prompt template from markdown file and format with variables
        prompt_template = ""
        if prompt_file:
            prompt_template = self.load_prompt_template(prompt_file)
        else:
            prompt_template = tool_agent_prompt
        
        if prompt_template == "":
            logger.error("No prompt template found")
            raise ValueError("No prompt template found")
            
        prompt = prompt_template.format(
            metadata=self.metadata,
            sample_records=sample_records,
            additional_instructions=additional_instructions
        )
        response = run_prompt(prompt)
        content = response["choices"][0]["message"]["content"]
        
        # Remove markdown code blocks if present
        if content.startswith("```json"):
            content = content.replace("```json", "").replace("```", "").strip()
        elif content.startswith("```"):
            content = content.replace("```", "").strip()
        
        try:
            return json.loads(content)
        except json.JSONDecodeError as e:
            logger.error(f"Failed to parse JSON response: {e}")
            logger.error(f"Raw content: {content}")
            raise ValueError(f"Invalid JSON response from LLM: {e}")

    def save_tool_to_file(self, tool_spec: Dict[str, Any], file_path: str) -> None:
        """Save tool specification to a file."""
        with open(file_path, 'w') as f:
            json.dump(tool_spec, f, indent=2)
        logger.info(f"Tool specification written to '{file_path}'")

def main():
    """Main function to parse arguments and generate tool specification."""
    parser = argparse.ArgumentParser(
        description="Generate tool specification for Astra DB table",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
Examples:
python generate_tool_spec.py --table-name users --db-name mydb --out-file user_tool.json
python generate_tool_spec.py -t products -d ecommerce -o product_tool.json
        """
    )

    parser.add_argument(
        "--export-prompt", "-ep",
        required=False,
        help="Export the prompt used to generate the tool specification"
    )
    
    parser.add_argument(
        "--table-name", "-t",
        required=False,
        help="Name of the table to analyze"
    )
    
    parser.add_argument(
        "--collection-name", "-c",
        required=False,
        help="Name of the collection to analyze"
    )
    
    parser.add_argument(
        "--keyspace-name", "-k",
        required=False,
        default="default_keyspace",
        help="Name of the keyspace to analyze"
    )
    
    parser.add_argument(
        "--db-name", "-d", 
        required=False,
        help="Name of the database containing the table"
    )
    
    parser.add_argument(
        "--out-file", "-o",
        required=False,
        help="Output file path for the generated tool specification JSON"
    )
    
    parser.add_argument(
        "--sample-size", "-s",
        type=int,
        default=5,
        help="Number of sample records to analyze (default: 5)"
    )
    
    parser.add_argument(
        "--prompt-file", "-pf",
        required=False,
        help="Prompt file to use for the tool specification generation"
    )
    
    parser.add_argument(
        "--additional-instructions", "-ai",
        required=False,
        help="Additional instructions for the tool specification generation"
    )
    
    args = parser.parse_args()
    
    if args.export_prompt:
        export_prompt(args.export_prompt)
        sys.exit(0)
        
    if (args.table_name and args.collection_name):
        logger.error("Either --table-name or --collection-name must be provided")
        sys.exit(1)
    
    if (not args.out_file):
        logger.error("--out-file is required")
        sys.exit(1)
 
    token = os.getenv("ASTRA_DB_APPLICATION_TOKEN")
    db_name = os.getenv("ASTRA_DB_DB_NAME")
    tool_agent = AstraToolAgent(token, db_name, args.table_name, args.keyspace_name)
    tool_spec = tool_agent.generate_tool_specification(args.sample_size, args.additional_instructions, args.prompt_file)
    if args.out_file:
        tool_agent.save_tool_to_file(tool_spec, args.out_file)
        logger.info(f"Tool specification written to '{args.out_file}'")
    else:
        logger.info("Tool specification:")
        logger.info(json.dumps(tool_spec, indent=2))
        sys.exit(1)

if __name__ == "__main__":
    main()



import argparse
import os
import json
from dotenv import load_dotenv
from fastmcp import FastMCP
from .load_tools import ToolLoader
from .database import AstraDBManager
from fastmcp.server.dependencies import get_context
from .logger import get_logger
from .run_tool import RunToolMiddleware
import uvicorn
import asyncio
from fastmcp.server.auth.providers.jwt import StaticTokenVerifier, TokenVerifier
from fastmcp.server.dependencies import get_http_headers
from .utils import load_env_variables

load_dotenv(override=True)

# Initialize logger
logger = get_logger("agentic_astra", level=os.getenv("LOG_LEVEL"), log_file=os.getenv("LOG_FILE"))

async def main():
    
    logger.info(f"Starting Astra MCP Server")
    
    # Parse arguments
    parser = argparse.ArgumentParser(conflict_handler="resolve")

    # ---- Uvicorn standard params ----
    parser.add_argument("--host", type=str, default="127.0.0.1",
                        help="Bind socket to this host")
    parser.add_argument("--port", type=int, default=8000,
                        help="Bind socket to this port")
    parser.add_argument("--reload", action="store_true",
                        help="Enable auto-reload")
    parser.add_argument("--workers", type=int, default=1,
                        help="Number of worker processes")
    parser.add_argument("--log-level", type=str,
                        default="info", help="Logging level")
    parser.add_argument("--log-file", type=str,
                        default="logs.log", help="Logging file")
    parser.add_argument("--transport", "-tr", default="stdio")

    # ---- Astra MCP Server params ----
    parser.add_argument("--astra_token", "-t",
                        default=os.getenv("ASTRA_DB_APPLICATION_TOKEN"))
    parser.add_argument("--astra_endpoint", "-e",
                        default=os.getenv("ASTRA_DB_API_ENDPOINT"))
    parser.add_argument("--astra_db_name", "-db",
                        default=os.getenv("ASTRA_DB_DB_NAME"))
    parser.add_argument("--catalog_file", "-f")
    parser.add_argument("--catalog_collection", "-c",
                        default=os.getenv("ASTRA_DB_CATALOG_COLLECTION") or "tool_catalog")
    parser.add_argument("--tags", "-tags")  # For filtering tools
    parser.add_argument("--auth", "-auth", default=True,
                        action="store_true", help="Disable authentication")
    parser.add_argument("--audit", "-audit", default=False,
                        action="store_true", help="Enable audit trail")
    parser.add_argument("--env", "-env", action="append", 
                        help="Environment variables in KEY=VALUE format (can be used multiple times)")

    args = parser.parse_args()
    
    # Load environment variables from command line arguments
    load_env_variables(args.env, logger)    
    
    astra_db_manager = AstraDBManager(
        token=args.astra_token or os.getenv("ASTRA_DB_APPLICATION_TOKEN"), 
        endpoint=args.astra_endpoint or os.getenv("ASTRA_DB_API_ENDPOINT"), 
        db_name=args.astra_db_name or os.getenv("ASTRA_DB_DB_NAME"))

    if args.audit:
        astra_db_manager.setup_audit_trail(os.getenv("ASTRA_DB_AUDIT_TABLE_NAME") or "mcp_audit_trail")

    # Initialize MCP
    # Configure JWT verifier
    token = os.getenv("AGENTIC_ASTRA_TOKEN") or os.getenv("ASTRA_MCP_SERVER_TOKEN")
    tokens_dict = {}
    if token:
        tokens_dict[token] = {
            "client_id": "agentic-astra",
            "scopes": ["read:data"]
        }
    verifier = StaticTokenVerifier(
        tokens=tokens_dict,
        required_scopes=["read:data"]
    )
    mcp = FastMCP("Astra MCP Server", auth=verifier)


    # Load tools config content
    tools_config_content = None
    if args.catalog_file:
        logger.info(f"Loading tools config from {args.catalog_file}")
        tools_config_content = json.load(open(args.catalog_file))
    else:
        logger.info(
            f"Loading tools Astra collection {args.catalog_collection}")
        tools_config_content = astra_db_manager.get_catalog_content(
            collection_name=args.catalog_collection, tags=args.tags)

    logger.info(f"Tools config content: {tools_config_content}")
    if not tools_config_content or len(tools_config_content) == 0:
        logger.error("No tools found. Load tools to Astra DB collection or reference the catalog file")
        return

    # Add middleware to process tool calling
    mcp.add_middleware(RunToolMiddleware(
        astra_db_manager, tools_config_content))

    logger.info("Initializing Astra MCP Server")
    logger.info(f"Starting Astra MCP Server on port {args.port}")

    # Generate tools based on tools config content
    tool_loader = ToolLoader(mcp, astra_db_manager, tools_config_content)
    tool_loader.load_all_tools()

    logger.info("All tools loaded successfully")

    app = None
    # Return the appropriate transport app
    if args.transport == "http" or args.transport == "sse":
        await mcp.run_async(transport=args.transport, host=args.host, port=args.port, log_level=args.log_level)
    elif args.transport == "stdio":
        await mcp.run_async(transport=args.transport, log_level=args.log_level)
    else:
        raise ValueError(f"Invalid transport: {args.transport}")
    logger.info("Astra MCP Server started successfully")


def run_server():
    """Synchronous entry point for the agentic-astra command."""
    asyncio.run(main())


if __name__ == "__main__":
    run_server()



from fastmcp.tools import Tool
from fastmcp import FastMCP, Context
from fastmcp.tools import FunctionTool
from fastmcp.tools.tool_transform import ArgTransform, forward_raw
from .database import AstraDBManager
from fastmcp.server.dependencies import get_context
from .logger import get_logger

class ToolLoader:
    def __init__(self, mcp: FastMCP, astra_db_manager: AstraDBManager,tools_config: dict):
        self.mcp = mcp
        self.astra_db_manager = astra_db_manager
        self.tools_config = tools_config
        self.logger = get_logger("tool_loader")
        self.tools = {}

    def load_all_tools(self):
        """Load all tools into the MCP server"""
        self.logger.info("Loading all tools into MCP server")
        self.load_database_tools()
        self.logger.info("All tools loaded successfully")

    def load_database_tools(self):
        """Load all database tools dynamically"""
        for tool_config in self.tools_config:
            tool = self.generate_tool(config=tool_config)
            self.mcp.add_tool(tool)

    @staticmethod   
    def generate_tool( config):
        """Generate a tool from a config"""
        
        parameters = {
            "type": "object",
            "required": []
        }
        
        if "parameters" in config:
            parameters["properties"] = {}
            
        for param in config["parameters"]:
            if "value" in param or "expr" in param: # If the parameter has a value or expression, it is not sent to the MCP Client
                continue
            
            parameters["properties"][param["param"] or param["attribute"]] = {
                "type": param.get("type", "string"),
                "description": param["description"],
            }
            
            if "enum" in param:
                parameters["properties"][param["param"]]["enum"] = param["enum"]
                
            if "required" in param:
                parameters["required"].append(param["param"])


        tool = Tool(
            name=config["name"],
            description=config["description"],
            parameters=parameters,
        )
        return tool




#!/usr/bin/env python3
"""
Setup script for agentic-astra package.
This setup.py works alongside pyproject.toml for PyPI publishing.
"""

from setuptools import setup, find_packages
import os

# Read the README file for long description
def read_readme():
    readme_path = os.path.join(os.path.dirname(__file__), "README.md")
    if os.path.exists(readme_path):
        with open(readme_path, "r", encoding="utf-8") as f:
            return f.read()
    return "Agentic Astra"

# Read the LICENSE file
def read_license():
    license_path = os.path.join(os.path.dirname(__file__), "LICENSE")
    if os.path.exists(license_path):
        with open(license_path, "r", encoding="utf-8") as f:
            return f.read()
    return "MIT"

setup(
    name="agentic-astra",
    version="0.0.5",
    description="Agentic Astra",
    long_description=read_readme(),
    long_description_content_type="text/markdown",
    author="Samuel Matioli",
    author_email="smatioli@gmail.com",
    maintainer="Samuel Matioli",
    maintainer_email="samuel.matioli@ibm.com",
    url="https://github.com/smatiolids/agentic-astra",
    project_urls={
        "Homepage": "https://github.com/smatiolids/agentic-astra",
        "Repository": "https://github.com/smatiolids/agentic-astra",
        "Issues": "https://github.com/smatiolids/agentic-astra/issues",
    },
    license=read_license(),
    license_files=["LICENSE"],
    keywords=["mcp", "astra", "datastax", "server"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.12",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Database",
        "Topic :: Internet :: WWW/HTTP :: HTTP Servers",
    ],
    python_requires=">=3.12",
    packages=find_packages(where="src", exclude=["tests*", "__pycache__*"]),
    package_dir={"": "src"},
    py_modules=[
        "agentic_astra.audit",
        "agentic_astra.auth",
        "agentic_astra.catalog",
        "agentic_astra.database", 
        "agentic_astra.llm",
        "agentic_astra.load_tools",
        "agentic_astra.logger",
        "agentic_astra.run_tool",
        "agentic_astra.server",
        "agentic_astra.tool_agent",
        "agentic_astra.tool_agent_prompt",
        "agentic_astra.utils",
    ],
    install_requires=[
        "astrapy>=2.0.1",
        "fastmcp>=2.12.1",
        "python-dotenv>=1.1.1",
        "uvicorn[standard]>=0.30.0",
    ],
    extras_require={
        "dev": [
            "pytest>=8.4.2",
            "pytest-asyncio>=1.2.0",
            "build>=1.3.0",
            "twine>=6.2.0",
        ]
    },
    entry_points={
        "console_scripts": [
            "agentic-astra=agentic_astra.server:run_server",
            "agentic-astra-catalog=agentic_astra.catalog:main",
        ],
    },
    include_package_data=True,
    zip_safe=False,
    # Additional metadata for PyPI
    download_url="https://github.com/smatiolids/agentic-astra/archive/v0.0.1.tar.gz",
    platforms=["any"],
    # Ensure compatibility with modern Python packaging
    setup_requires=["setuptools>=45", "wheel"],
)
